from sqlalchemy import \
    Column,\
    BigInteger,\
    Boolean,\
    Integer,\
    Interval,\
    String,\
    Numeric,\
    DateTime,\
    JSON,\
    Float,\
    Text,\
    ForeignKey,\
    MetaData,\
    DDL,\
    event
from sqlalchemy.types import ARRAY
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import JSONB

import os


Base = declarative_base(metadata=MetaData(schema='results'))
event.listen(
    Base.metadata,
    'before_create',
    DDL("CREATE SCHEMA IF NOT EXISTS results")
)

group_proc_filename = os.path.join(
    os.path.dirname(__file__),
    'model_group_stored_procedure.sql'
)
with open(group_proc_filename) as f:
    stmt = f.read()

event.listen(
    Base.metadata,
    'before_create',
    DDL(stmt)
)


class Experiment(Base):
    __tablename__ = 'experiments'
    experiment_hash = Column(String, primary_key=True)
    config = Column(JSONB)


class ModelGroup(Base):
    __tablename__ = 'model_groups'
    model_group_id = Column(Integer, primary_key=True)
    model_type = Column(Text)
    model_parameters = Column(JSONB)
    feature_list = Column(ARRAY(Text))
    model_config = Column(JSONB)


class Model(Base):
    __tablename__ = 'models'
    model_id = Column(Integer, primary_key=True)
    model_group_id = Column(Integer, ForeignKey('model_groups.model_group_id'))
    model_hash = Column(String, unique=True, index=True)
    run_time = Column(DateTime)
    batch_run_time = Column(DateTime)
    model_type = Column(String)
    model_parameters = Column(JSONB)
    model_comment = Column(Text)
    batch_comment = Column(Text)
    config = Column(JSON)
    experiment_hash = Column(String, ForeignKey('experiments.experiment_hash'))
    train_end_time = Column(DateTime)
    test = Column(Boolean)
    train_matrix_uuid = Column(Text)
    train_label_window = Column(Interval)

    model_group_rel = relationship('ModelGroup')
    experiment_rel = relationship('Experiment')

    def delete(self, session):
        # basically implement a cascade, in case cascade is not implemented
        session.query(FeatureImportance)\
            .filter_by(model_id=self.model_id)\
            .delete()
        session.query(Evaluation)\
            .filter_by(model_id=self.model_id)\
            .delete()
        session.query(Prediction)\
            .filter_by(model_id=self.model_id)\
            .delete()
        session.delete(self)


class FeatureImportance(Base):
    __tablename__ = 'feature_importances'
    model_id = Column(Integer, ForeignKey('models.model_id'), primary_key=True)
    model = relationship(Model)
    feature = Column(String, primary_key=True)
    feature_importance = Column(Numeric)
    rank_abs = Column(Integer)
    rank_pct = Column(Float)

    model_rel = relationship('Model')

class Prediction(Base):
    __tablename__ = 'predictions'
    model_id = Column(Integer, ForeignKey('models.model_id'), primary_key=True)
    entity_id = Column(BigInteger, primary_key=True)
    as_of_date = Column(DateTime, primary_key=True)
    score = Column(Numeric)
    label_value = Column(Integer)
    rank_abs = Column(Integer)
    rank_pct = Column(Float)
    matrix_uuid = Column(Text)
    test_label_window = Column(Interval)

    model_rel = relationship('Model')

class Evaluation(Base):
    __tablename__ = 'evaluations'
    model_id = Column(Integer, ForeignKey('models.model_id'), primary_key=True)
    evaluation_start_time = Column(DateTime, primary_key=True)
    evaluation_end_time = Column(DateTime, primary_key=True)
    example_frequency = Column(Interval, primary_key=True)
    metric = Column(String, primary_key=True)
    parameter = Column(String, primary_key=True)
    value = Column(Numeric)
    num_labeled_examples = Column(Integer)
    num_labeled_above_threshold = Column(Integer)
    num_positive_labels = Column(Integer)
    sort_seed = Column(Integer)

    model_rel = relationship('Model')



from __future__ import with_statement
from alembic import context
from sqlalchemy import pool
from logging.config import fileConfig
from results_schema import Base
from sqlalchemy.engine.url import URL
from sqlalchemy import create_engine
import os
import yaml


# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.

url = os.environ.get('DBURL', None)

if not url:
    db_config_file = context.get_x_argument('db_config_file')\
        .get('db_config_file', None)
    if not db_config_file:
        raise ValueError('No database connection information found')

    with open(db_config_file) as f:
        config = yaml.load(f)
        dbconfig = {
            'host': config['host'],
            'username': config['user'],
            'database': config['db'],
            'password': config['pass'],
            'port': config['port'],
        }
        url = URL('postgres', **dbconfig)


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        version_table='results_schema_versions'
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """

    connectable = create_engine(
        url,
        poolclass=pool.NullPool
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            version_table='results_schema_versions'
        )

        with context.begin_transaction():
            context.run_migrations()

if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
